// ****************************************************************************
// copyright (c) 2000-2004 Horst Knorr <hk_classes@knoda.org>  
// This file is part of the hk_kdeclasses library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file LGPL included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
//$Revision: 1.6 $

#include "hk_kdedbdesigner.h"
#include <hk_presentation.h>
#include <hk_datasource.h>
#include <hk_dsdatavisible.h>
#include <list>
#include <qscrollview.h>
#include <qlistbox.h>
#include <qlayout.h>
#include <qcolor.h>
#include <qpalette.h>
#include <qpainter.h>
#include <qlabel.h>
#include <qptrlist.h>
#include <qbitmap.h>




class hk_kdedbdesignerprivate
{
public:
hk_kdedbdesignerprivate( hk_kdedbdesigner* designer)
	{
		p_designer=designer;
		p_layout=new QVBoxLayout(designer);
		p_scrollview=new QScrollView(designer);
		p_layout->addWidget(p_scrollview);
		p_scrollview->viewport()->setPaletteBackgroundColor(p_scrollview->colorGroup().mid());
		p_presentation=NULL;
		p_datasources=NULL;
	}
virtual ~hk_kdedbdesignerprivate()
	{

	}
QVBoxLayout*	p_layout;
QScrollView* p_scrollview;
hk_kdedbdesigner* p_designer;
hk_presentation* p_presentation;
		list<hk_datasource*>* p_datasources;
list<hk_kdedatasourceframe*> p_windowlist;
};




kdedatasourcelabel::kdedatasourcelabel(QWidget* w):QLabel(w)
	{
		p_drag=false;
		p_startx=p_starty=p_offsetx=p_offsety=0;
		installEventFilter(this);
	setPaletteBackgroundColor(palette().active().highlight());
	setPaletteForegroundColor(palette().active().highlightedText());
	}


bool  kdedatasourcelabel::eventFilter(QObject* /*object*/, QEvent* event)
	{

	  if (event->type()==QEvent::MouseMove)
	   {
	if (p_drag && static_cast<QMouseEvent*>(event)->state()==Qt::LeftButton)
	  {
		int xdiff=static_cast<QMouseEvent*>(event)->globalPos().x()-p_startx;
		int ydiff=static_cast<QMouseEvent*>(event)->globalPos().y()-p_starty;
		if ((abs(xdiff)>2) || (abs(ydiff)>2))
		{
			QPoint newPos=parentWidget()->pos()+QPoint(xdiff,ydiff);
			if (newPos.x()<0)
			{
				p_offsetx+=newPos.x();
				newPos.setX(0);
			}
			else if (p_offsetx<0)
			{
				p_offsetx+=newPos.x();
				if (p_offsetx>0)
				{
					newPos.setX(p_offsetx);
					p_offsetx=0;
				}
				else newPos.setX(0);
			}
			if (newPos.y()<0)
			{
				p_offsety+=newPos.y();
				newPos.setY(0);
			}
			else
			if (p_offsety<0)
			{
				p_offsety+=newPos.y();
				if (p_offsety>0)
				{
					newPos.setY(p_offsety);
					p_offsety=0;
				}
				else newPos.setY(0);
			}
			parentWidget()->move(newPos);
			p_startx=static_cast<QMouseEvent*>(event)->globalPos().x();
			p_starty=static_cast<QMouseEvent*>(event)->globalPos().y();
   		}

	return true;
   	}
   }
return false;
}


void kdedatasourcelabel::mousePressEvent(QMouseEvent* event)
	{
	if (parentWidget())
		parentWidget()->setFocus();
	if (event->button()==Qt::LeftButton)
	   {
		p_drag=true;
		p_startx=event->globalPos().x();
		p_starty=event->globalPos().y();
		p_offsetx=p_offsety=0;
	   }

	}


void kdedatasourcelabel::mouseReleaseEvent(QMouseEvent* event)
	{
	if (event->button()==Qt::LeftButton)
	   {
		p_drag=false;
	   }

	}






hk_kdefieldlist::hk_kdefieldlist(QWidget* parent):QListBox(parent)
	{

	};

void hk_kdefieldlist::setFocus(void)
	{
		parentWidget()->setFocus();
		QListBox::setFocus();
	}

void hk_kdefieldlist::clearFocus(void)
	{
		parentWidget()->clearFocus();
		QListBox::clearFocus();
	};






hk_kdedatasourceframe::hk_kdedatasourceframe(hk_kdedbdesigner* designer,QWidget* parent,hk_datasource* ds):
	QFrame(parent)
	,hk_dsdatavisible()
	{
	  setFrameStyle( QFrame::WinPanel | QFrame::Raised );
	  p_layout=new QVBoxLayout(this,2,1);
	  p_listbox=new hk_kdefieldlist(this);
		p_designer = designer;
		p_positionupdate=true;
		if (ds)
		{
			setGeometry(ds->x(),ds->y(),ds->width(),ds->height());
		}
		else 	setGeometry(20,20,150,150);
		p_positionupdate=false;
		set_datasource(ds);
	 p_header=new kdedatasourcelabel(this);
		set_fields();
	  p_layout->addWidget(p_header);
	  p_layout->addWidget(p_listbox);

	}


void hk_kdedatasourceframe::set_fields(void)
	{
	  if (!datasource()) return;
	  list<hk_column*>* cols=datasource()->columns();
	  if (!cols) return;
	  list<hk_column*>::iterator it=cols->begin();
	  while (it!=cols->end())
	  {
	  	p_listbox->insertItem(QString::fromLocal8Bit((*it)->name().c_str()));
		++it;
	  }
	 p_header->setText(QString::fromLocal8Bit(datasource()->name().c_str()));
	}




void hk_kdedatasourceframe::setFocus(void)
	{
		p_header->setFocus();
		QFrame::setFocus();
	}

void hk_kdedatasourceframe::clearFocus(void)
	{
		p_header->clearFocus();
		QFrame::clearFocus();

	}

void hk_kdedatasourceframe::mousePressEvent(QMouseEvent*)
{

}

void hk_kdedatasourceframe::moveEvent(QMoveEvent*)
{
hk_datasource* ds=datasource();
if (!ds||p_positionupdate) return;
ds->set_position(QWidget::x(),QWidget::y(),false);
ds->set_size(QWidget::width(),QWidget::height(),false);
emit signal_moved();
}






 hk_kdedbdesigner::hk_kdedbdesigner(void):QWidget(),hk_dbvisible()
{
  p_private=new hk_kdedbdesignerprivate(this);
  setGeometry(QWidget::x(),QWidget::y(),600,500);
}



hk_kdedbdesigner::~hk_kdedbdesigner()
{
delete p_private->p_layout;
delete p_private->p_scrollview;
delete p_private;
}


void hk_kdedbdesigner::set_presentation(hk_presentation * p)
{
p_private->p_presentation =p;
p_private->p_datasources=NULL;
clear_datasources();
if (!p_private->p_presentation) return;
p_private->p_datasources=p_private->p_presentation->datasources();
set_datasources();
}

hk_presentation* hk_kdedbdesigner::presentation(void)
{
return p_private->p_presentation;
}



void hk_kdedbdesigner::clear_datasources()
{
 if (!p_private->p_datasources)return;
}


hk_kdedatasourceframe* hk_kdedbdesigner::get_frame(hk_datasource* ds)
{
  if (!ds) return NULL;
  list<hk_kdedatasourceframe*>::iterator it=p_private->p_windowlist.begin();
  cerr <<"get_frame begin"<<endl;
while (it!=p_private->p_windowlist.end())
  	{
	   cerr <<"ds="<<ds<<" f->datasource= "<<(*it)->datasource()<<endl;
	   if ((*it)->datasource()==ds)
	   	{
		  cerr <<"get_frame ds gefunden"<<endl;
		  return (*it);
		}
	   ++it;
	}
  cerr <<"get_frame end"<<endl;
return NULL;
}






void hk_kdedbdesigner::set_datasources()
{
 if (!p_private->p_datasources)
 	{
	 cerr <<"p_private->p_datasources="<<p_private->p_datasources<<endl;
	 return;
	}
int pos=5;
 list<hk_datasource*>::iterator it=p_private->p_datasources->begin();
 while (it!=p_private->p_datasources->end())
 {
  hk_kdedatasourceframe* k=new hk_kdedatasourceframe(this,p_private->p_scrollview, (*it));
  k->set_datasource((*it));
  k->move(pos,20);
  if ((*it)->name().size()>0)
  	p_private->p_windowlist.insert(p_private->p_windowlist.end(),k);
  ++it;
 pos+=k->QWidget::width()+20;
 }

cerr <<"create relation begin"<<endl;

  list<hk_kdedatasourceframe*>::iterator sit=p_private->p_windowlist.begin();
while (sit!=p_private->p_windowlist.end())
  	{
	   hk_datasource* ds=(*sit)->datasource(); cerr <<"setze Datenquelle ="<<ds->name()<<endl;
	   if (ds && ds->depending_on())
	   {
		hk_kdedbrelation* relation=new hk_kdedbrelation(p_private->p_scrollview);
  		relation->setAutoMask(true);
		cerr <<"f="<<(*sit)<<" dep "<<ds->depending_on()<<endl;
		relation->set_datasources(get_frame(ds->depending_on()),(*sit));
  		relation->setGeometry(10,50,100,70);
	   }
	   else
	      cerr <<"kein relation, da ds=="<<ds<<" und depon=="<<ds->depending_on()<<endl;
	++sit;
	}
cerr <<"create relation end"<<endl;




sit=p_private->p_windowlist.begin();
while (sit!=p_private->p_windowlist.end())
  	{
	  (*sit)->raise();
	  ++sit;
	}

}











// hk_kdeDBRELATION BEGIN
  const  int hlength=10;//horizontal line length
  const  int  arrowlength=5;
  const  int  arrowheight=5;

hk_kdedbrelation::hk_kdedbrelation(QWidget* parent):QWidget(parent),hk_class()
{
p_tox=p_toy=p_fromx=p_fromy=0;
}

hk_kdedbrelation::~hk_kdedbrelation()
{
}

void hk_kdedbrelation::paint_relation(QPainter* paint)
{
   paint->save();
/*   paint->drawLine(0,0,width(),0);
   paint->drawLine(0,height()-1,width(),height()-1);
   paint->drawLine(0,0,0,height());
   paint->drawLine(width()-1,0,width()-1,height());
*/

   paint->drawLine(p_fromx+hlength,p_fromy,p_tox-hlength,p_toy+arrowheight); //diagonal

    paint->drawLine(p_fromx,p_fromy,p_fromx+hlength,p_fromy);//master horizontal
   paint->drawLine(p_tox-hlength,p_toy+arrowheight,  p_tox ,p_toy+arrowheight);//slave horizontal
   QPointArray array;
   array.setPoints(3,p_tox-1,p_toy+arrowheight, p_tox-arrowlength-1,p_toy,p_tox-arrowlength-1,p_toy+2*arrowheight);
   paint->drawPolygon(array);
   cerr <<"paint fx= "<<p_fromx<<" fy= "<<p_fromy<<" tx= "<<p_tox<<" ty= "<<p_toy<<endl;
   cerr <<"slave fx= "<<p_tox-hlength<<" fy= "<<p_toy-1<<" tx= "<<p_tox<<" ty= "<<p_toy-1<<endl<<endl;

   paint->restore();
}



void hk_kdedbrelation::paintEvent(QPaintEvent*)
{
  if (autoMask()) return;
  QPainter paint(this);
  paint.setBrush(colorGroup().foreground());
  paint_relation(&paint);
}


void hk_kdedbrelation::updateMask(void)
{
   QBitmap bm(size());
   bm.fill(color0);
   QPainter paint;
   paint.begin(&bm,this);
   paint.setBrush(color1);
   paint.setPen(color1);
   paint_relation(&paint);
   paint.end();
   setMask(bm);
}

void  hk_kdedbrelation::setAutoMask(bool b)
{

	setBackgroundMode(b?PaletteForeground:PaletteBackground);
	QWidget::setAutoMask(b);

}


void hk_kdedbrelation::set_datasources(hk_kdedatasourceframe* master,hk_kdedatasourceframe* slave)
{
if (!master||!slave) return;


 p_masterdatasource=master;
 p_slavedatasource=slave;

 connect(p_masterdatasource,SIGNAL(signal_moved()),this,SLOT(datasource_moved()));
 connect(p_slavedatasource,SIGNAL(signal_moved()),this,SLOT(datasource_moved()));
 datasource_moved();
}

hk_kdedatasourceframe* hk_kdedbrelation::masterdatasource(void) const
{
  return p_masterdatasource;
}


hk_kdedatasourceframe* hk_kdedbrelation::slavedatasource(void) const
{
  return p_slavedatasource;
}



void hk_kdedbrelation::datasource_moved(void)
{
int x,y,w,h,mx,my,mh,sx,sy,sh;
mx=p_masterdatasource->QWidget::x()+p_masterdatasource->QWidget::width();
sx=p_slavedatasource->QWidget::x();
if (mx<sx)
{ //standard case;
   x=mx;
   w=sx-mx;
   p_fromx=0;
   p_tox=w;cerr<<"sh ";
}
else
{cerr<<"nh ";
   x=sx-hlength;
   w=mx-sx+2*hlength;
   p_fromx=w-hlength;
   p_tox=hlength;

}
if (w<hlength)w=hlength;

my=p_masterdatasource->QWidget::y();
mh=p_slavedatasource->QWidget::y()-p_masterdatasource->QWidget::y();
sy=p_slavedatasource->QWidget::y();
sh=-mh;
if (my<sy)
{//standard case
cerr <<" sv";
y=my;
h=mh;
p_fromy=0;
p_toy=h;
}
else
{
cerr <<" nv";
y=sy;
h=sh;
if (h>0)p_fromy=h;
else p_fromy=0;
p_toy=0;
}

cerr <<endl;
if (h<3)h=3;
if (w<3)w=3;

setGeometry(x,y+5,w,h+10);

}
